-- **************************************************

-- Provide Moho with the name of this script object

-- **************************************************


ScriptName = "MG_AnimBake"

-- **************************************************

-- General information about this script

-- **************************************************


MG_AnimBake = {}

--MG_AnimBake.BASE_STR = 2495

function MG_AnimBake:Name()
    return "MG Animation Baker"
end

function MG_AnimBake:Version()
    return "5.0"
end

function MG_AnimBake:Description()
    return "Bakes a keyframe onto every frame"
end

function MG_AnimBake:Creator()
    return "Mike Green"
end

function MG_AnimBake:UILabel()
    return("Animation Baker")
end

-- **************************************************

-- Recurring values

-- **************************************************

MG_AnimBake.frameStep = 1
MG_AnimBake.startFrame = 1
MG_AnimBake.endFrame = 1
MG_AnimBake.layerTrans = true
MG_AnimBake.layerScale = true
MG_AnimBake.layerRotat = true
MG_AnimBake.boneTrans = true
MG_AnimBake.boneScale = true
MG_AnimBake.boneRotat = true


-- **************************************************
-- Motion bake dialog
-- **************************************************

local MG_AnimBakeDialog = {}

function MG_AnimBakeDialog:new(moho)
	local d = LM.GUI.SimpleDialog("Motion Baker", MG_AnimBakeDialog)
	local l = d:GetLayout()

	d.moho = moho

	l:PushH()
		l:PushV()
			l:AddChild(LM.GUI.StaticText("Frame step"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Start frame"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("End frame"), LM.GUI.ALIGN_LEFT)
		l:Pop()
		l:PushV()
			d.frameStep = LM.GUI.TextControl(0, "0.0000", 0, LM.GUI.FIELD_UINT)
			l:AddChild(d.frameStep)
			d.startFrame = LM.GUI.TextControl(0, "0.0000", 0, LM.GUI.FIELD_UINT)
			l:AddChild(d.startFrame)
			d.endFrame = LM.GUI.TextControl(0, "0.0000", 0, LM.GUI.FIELD_UINT)
			l:AddChild(d.endFrame)
		l:Pop()
	l:Pop()

	d.layerTrans = LM.GUI.CheckBox("Layer Translate")
	l:AddChild(d.layerTrans, LM.GUI.ALIGN_LEFT)
	d.layerScale = LM.GUI.CheckBox("Layer Scale")
	l:AddChild(d.layerScale, LM.GUI.ALIGN_LEFT)
	d.layerRotat = LM.GUI.CheckBox("Layer Rotate")
	l:AddChild(d.layerRotat, LM.GUI.ALIGN_LEFT)

	d.boneTrans = LM.GUI.CheckBox("Bone Translate")
	l:AddChild(d.boneTrans, LM.GUI.ALIGN_LEFT)
	d.boneScale = LM.GUI.CheckBox("Bone Scale")
	l:AddChild(d.boneScale, LM.GUI.ALIGN_LEFT)
	d.boneRotat = LM.GUI.CheckBox("Bone Rotate")
	l:AddChild(d.boneRotat, LM.GUI.ALIGN_LEFT)

	return d
end

function MG_AnimBakeDialog:UpdateWidgets()

	self.frameStep:SetValue(MG_AnimBake.frameStep)
	self.startFrame:SetValue(MG_AnimBake.startFrame)
	self.endFrame:SetValue(MG_AnimBake.endFrame)

	self.layerTrans:SetValue(MG_AnimBake.layerTrans)
	self.layerScale:SetValue(MG_AnimBake.layerScale)
	self.layerRotat:SetValue(MG_AnimBake.layerRotat)

	self.boneTrans:SetValue(MG_AnimBake.boneTrans)
	self.boneScale:SetValue(MG_AnimBake.boneScale)
	self.boneRotat:SetValue(MG_AnimBake.boneRotat)
end

function MG_AnimBakeDialog:OnValidate()
	local b = true
	if (not self:Validate(self.startFrame, 0, 1000000)) then
		b = false
	end
	if (not self:Validate(self.endFrame, self.startFrame:IntValue() + 1, 1000000)) then
		b = false
	end
	return b
end

function MG_AnimBakeDialog:OnOK()
	MG_AnimBake.frameStep = self.frameStep:IntValue()
	MG_AnimBake.startFrame = self.startFrame:IntValue()
	MG_AnimBake.endFrame = self.endFrame:IntValue()

	MG_AnimBake.layerTrans = self.layerTrans:Value()
	MG_AnimBake.layerScale = self.layerScale:Value()
	MG_AnimBake.layerRotat = self.layerRotat:Value()

	MG_AnimBake.boneTrans = self.boneTrans:Value()
	MG_AnimBake.boneScale = self.boneScale:Value()
	MG_AnimBake.boneRotat = self.boneRotat:Value()
end


-- **************************************************

-- The guts of this script

-- **************************************************


function MG_AnimBake:Run(moho)
	MG_AnimBake.endFrame = moho.document:AnimDuration()
	currentlayer=moho.document:Layer(0)
	moho.document:PrepUndo(currentlayer)

	local dlog = MG_AnimBakeDialog:new(moho)
	if (dlog:DoModal() == LM.GUI.MSG_CANCEL) then
		return
	end

	local mylayercount=moho.document:CountLayers()
	for i=0, mylayercount-1, 1 do
		local mylayer=moho.document:Layer(i)
		MG_AnimBake:BakeLayerMotion(moho,mylayer)
	end
--	print( "Number of layers:"..mylayercount)
end

function MG_AnimBake:BakeLayerMotion(moho,layer)
	local mytrans = {}
	local myscale = {}
	local myrotx = {}
	local myroty = {}
	local myrotz = {}
	for i=MG_AnimBake.startFrame, MG_AnimBake.endFrame, MG_AnimBake.frameStep do
    	mytrans[i] = layer.fTranslation:GetValue(i)
    	myscale[i]=layer.fScale:GetValue(i)
    	myrotx[i]=layer.fRotationX:GetValue(i)
    	myroty[i]=layer.fRotationY:GetValue(i)
    	myrotz[i]=layer.fRotationZ:GetValue(i)
	end
	for i=MG_AnimBake.startFrame, MG_AnimBake.endFrame, MG_AnimBake.frameStep do
		moho:SetCurFrame(i);
		if MG_AnimBake.layerTrans then
        	layer.fTranslation:SetValue(i,mytrans[i])
	    	layer.fTranslation:StoreValue()
	    end
		if MG_AnimBake.layerScale then
        	layer.fScale:SetValue(i,myscale[i])
	    	layer.fScale:StoreValue()
	    end
		if MG_AnimBake.layerRotat then
        	layer.fRotationX:SetValue(i,myrotx[i])
	    	layer.fRotationX:StoreValue()
        	layer.fRotationY:SetValue(i,myroty[i])
	    	layer.fRotationY:StoreValue()
        	layer.fRotationZ:SetValue(i,myrotz[i])
	    	layer.fRotationZ:StoreValue()
	    end
	end

	if layer:IsGroupType() then
		local mygrouplayer=moho:LayerAsGroup(layer)
		local mylayercount = mygrouplayer:CountLayers()
		for i=0, mylayercount-1, 1 do
			local mylayer=mygrouplayer:Layer(i)
			MG_AnimBake:BakeLayerMotion(moho,mylayer)
		end
--		print("Is a group")
	end

	if layer:IsBoneType() then
		moho:SetSelLayer(layer)
		local mybonelayer=moho:Skeleton()
		local mybonecount = mybonelayer:CountBones()
		for i=0, mybonecount-1, 1 do
			local mybone=mybonelayer:Bone(i)
			MG_AnimBake:BakeBoneMotion(moho,mybone)
		end
	end
--	print("Baking Layer:"..layer:Name())
end

function MG_AnimBake:BakeBoneMotion(moho,bone)
	local animlength = moho.document:AnimDuration()
	local mytrans = {}
	local myangle = {}
	local myscale = {}
	for i=MG_AnimBake.startFrame, MG_AnimBake.endFrame, MG_AnimBake.frameStep do
    	mytrans[i] = bone.fAnimPos:GetValue(i)
    	myangle[i]= bone.fAnimAngle:GetValue(i)
    	myscale[i]= bone.fAnimScale:GetValue(i)
	end
	for i=MG_AnimBake.startFrame, MG_AnimBake.endFrame, MG_AnimBake.frameStep do
		moho:SetCurFrame(i);
		if MG_AnimBake.layerTrans then
        	bone.fAnimPos:SetValue(i,mytrans[i])
	    	bone.fAnimPos:StoreValue()
	    end
		if MG_AnimBake.layerRotat then
        	bone.fAnimAngle:SetValue(i,myangle[i])
	    	bone.fAnimAngle:StoreValue()
	    end
	    if MG_AnimBake.layerScale then
        	bone.fAnimScale:SetValue(i,myscale[i])
	    	bone.fAnimScale:StoreValue()
	    end
	end
end